@@ -36,7 +36,7 @@ from gevent.event import AsyncResult
 
 from logging import root as logger, currentframe
 
-
+DEBUG = 1
 PICKLE_PROTOCOL = HIGHEST_PROTOCOL
 class RpcError(StandardError):
     pass
@@ -104,9 +104,13 @@ def printf(msg, *args):
         print('%s%s' % (log_head, msg % args))
 
 
-def log_except(clear=False):
+def log_except(*args, **kw):
     ei = sys.exc_info()
-    lines = ['Traceback (most recent call last):\n']
+    if args:
+        lines = [agrs[0] % args[1:], '\n']
+    else:
+        lines = []
+    lines.append('Traceback (most recent call last):\n')
     st = traceback.extract_stack(f=ei[2].tb_frame.f_back)
     et = traceback.extract_tb(ei[2])
     lines.extend(traceback.format_list(st))
@@ -114,7 +118,7 @@ def log_except(clear=False):
     lines.extend(traceback.format_list(et))
     lines.extend(traceback.format_exception_only(ei[0], ei[1]))
     exc = ''.join(lines)
-    if clear:
+    if kw.get('clear', False):
         sys.exc_clear()
     printf(exc)
 
@@ -259,7 +263,6 @@ class RpcBase(object):
             pass
         self._socket.close()
 
-
     def after_start(self):
         pass
 
@@ -281,7 +284,6 @@ class RpcBase(object):
         finally:
             self.close()
 
-
     def _handle(self, parts):
         """ 单个数据包处理 """
         pass
@@ -367,8 +369,11 @@ class RpcService(object):
         self._stop_resps()
         self._stop_proxys()
         if not self.sock_error:
-            self.call(0, 'remote_stop', tuple(), None, no_result=True)
-            sleep(0.1)
+            try:
+                self.call(0, 'remote_stop', tuple(), None, no_result=True)
+                sleep(0.05)
+            except:
+                pass
         if getattr(self, '_heart_task', None):
             self._heart_task.kill(block=False)
             self._heart_task = None
@@ -589,7 +594,7 @@ class RpcService(object):
             else:
                 self.send(RT_RESPONSE | DT_PICKLE, index, pickle_dumps(rs, PICKLE_PROTOCOL))
         except Exception as e:
-            log_except()
+            log_except('export(%s).%s(%s)', obj_id, name, argkw)
             if dtype & ST_NO_RESULT or self.svr.stoped:
                 return
             self.send(RT_EXCEPTION, index, pickle_dumps(e, PICKLE_PROTOCOL))
@@ -767,9 +772,9 @@ class RpcServer(RpcBase):
     def before_stop(self):
         if self.addr in _my_servers_:
             _my_servers_.pop(self.addr, None)
+        self._accept_task.kill(block=False)
         for svc in self._services.values():
             svc.stop()
-        self._accept_task.kill(block=True)
 
     def get_service(self, uid):
         try:
@@ -819,7 +824,8 @@ class RpcServer(RpcBase):
 #import pickle
 def proxy_reconstructor(proxy_cls, addr, export_id, timeout):
     proxy = get_proxy_by_addr(addr, export_id, proxy_cls=proxy_cls)
-    proxy.timeout = timeout
+    if proxy:
+        proxy.timeout = timeout
     return proxy
 
 class RpcProxy(object):
@@ -997,6 +1003,18 @@ def map_items_ex(proxys, attr, *args, **kw):
 
 _my_servers_ = WeakValueDictionary()
 _addr_clients = WeakValueDictionary()
+def uninit():
+    """ clear all """
+    global _my_servers_, _addr_clients
+    clients = _addr_clients.values()
+    _addr_clients.clear()
+    servers = _my_servers_.values()
+    _my_servers_.clear()
+    for cli in clients:
+        cli.stop()
+    for svr in servers:
+        svr.stop()
+
 def get_rpc_by_addr(addr, force_rpc=0):
     global _addr_clients, _my_servers_
     if isinstance(addr, list):
